# -*- coding: utf-8 -*-
#
##########################################################################
# creole.containers - management of LXC containers
# Copyright © 2012,2013 Pôle de compétences EOLE <eole@ac-dijon.fr>
#
# License CeCILL:
#  * in french: http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
#  * in english http://www.cecill.info/licences/Licence_CeCILL_V2-en.html
##########################################################################

"""Manage LXC containers

"""

from creole.client import CreoleClient, _CONTAINER_COMPONENTS
from creole.config import VIRTENABLED_LOCKFILE, VIRTDISABLED_LOCKFILE
from creole.error import VirtError
from creole.config import templatedir, VIRTROOT
from creole.template import CreoleTemplateEngine
from pyeole.process import system_code, system_out, system_progress_out
from pyeole.diagnose import test_tcp
from creole.i18n import _

from distutils.spawn import find_executable
from os.path import isdir, isfile, join
from os import symlink
from os import makedirs
from hashlib import md5
from glob import glob
import cjson

client = CreoleClient()

_LXC_MD5 = '/etc/eole/lxc.md5'

_NOT_REALLY_LXC_CONTAINERS = ['root', 'all']
"""List of container names that are not to be generated.

"""

_LXC_TEMPLATE = {'config': "lxc.config",
                 'fstab': "lxc.fstab",
                 'rootfs/etc/network/interfaces' : "lxc.interfaces",
                }
"""Creole templates for LXC containers.

"""

def is_lxc_locked():
    """Check if the LXC virtualization is locked.

    The virtualization is locked after first ``instance`` of the
    server to avoid switching between modes.

    :return: ``enable`` if LXC is enabled, ``disable`` if LXC is
             disabled or ``None`` where there is no lockfile.

    """
    if isfile(VIRTENABLED_LOCKFILE) and isfile(VIRTDISABLED_LOCKFILE):
        raise VirtError(_(u"Invalid LXC lock files state: both are present."))
    elif isfile(VIRTENABLED_LOCKFILE):
        virtlocked = 'enable'
    elif isfile(VIRTDISABLED_LOCKFILE):
        virtlocked = 'disable'
    else:
        virtlocked = None
    return virtlocked

def is_lxc_enabled():
    """Check if LXC controller is enabled

    We do not accept to switch between enabled and disabled LXC, after
    first ``instance``, a lock file is set to check at each
    ``reconfigure``.

    :return: If the LXC container mode is enabled.
    :rtype: `bool`
    :raise VirtError: if state in inconsistent between configuration
                      and lock files.

    """
    containers_enabled = client.get_creole('mode_conteneur_actif', 'non') == 'oui'
    if containers_enabled and not find_executable('lxc-info'):
        raise VirtError(_(u'LXC is enabled but LXC commands not found in PATH.'))

    if containers_enabled and is_lxc_locked() == 'disable':
        raise VirtError(_(u"Server already instantiated in no containers mode, attempt to activate containers mode aborted."))
    elif not containers_enabled and is_lxc_locked() == 'enable':
        raise VirtError(_(u"Server already instantiated in containers mode, attempt to activate no containers mode aborted."))

    return containers_enabled

def generate_lxc_container(name, logger=None):
    """Run creation of a container.

    Check if LXC is enabled and take care of ``root`` and ``all``
    containers.

    :param name: name of the LXC container
    :type name: `str`

    """
    if name not in _NOT_REALLY_LXC_CONTAINERS:
        cmd = ['lxc-create', '-n', name,
               '-t', 'eole', '--',
               '--release', 'precise']
        code, stdout, stderr = system_progress_out(cmd, _(u"Managing container {0}").format(name), logger)
        if code != 0 and stdout.find(u"'{0}' already exists'".format(name)) >= 0:
            raise Exception(_('error during the process of container creation'))

        auto_file = '/etc/lxc/auto/{0}.conf'.format(name)
        if not isfile(auto_file):
            symlink('/var/lib/lxc/{0}/config'.format(name), auto_file)


def is_lxc_running(container):
    """Check if an LXC container is running.

    This check at LXC level and check TCP on port SSH.

    :param container: the container informations
    :type container: `dict`
    :return: if the container is running and reachable
    :rtype: `bool`

    """

    if container.get(u'name', None) is None:
        raise ValueError(_(u"Container has no name"))

    if container.get(u'ip', None) is None:
        raise ValueError(_(u"Container {0} has no IP").format(container[u'name']))

    cmd = ['lxc-info', '--state', '--name', container[u'name']]
    code, stdout, stderr = system_out(cmd)

    return stdout.strip().endswith('RUNNING') and test_tcp(container[u'ip'], 22)


def create_mount_point(group):
    """Create mount points in LXC.

    This is required for LXC to start.

    """
    if 'fstabs' not in group:
        return
    for fstab in group['fstabs']:
        full_path = join(group['path'], fstab['mount_point'].lstrip('/'))
        if not isdir(full_path):
            makedirs(full_path)


def lxc_need_restart():
    def md5sum(file):
        return md5(open(file).read()).hexdigest()
    files = ['/etc/lxc/lxc.conf', '/etc/default/lxc']
    files += glob('/opt/lxc/*/config')
    files += glob('/opt/lxc/*/fstab')
    md5s = []
    for f in files:
        md5s.append(md5sum(f))
    if not isfile(_LXC_MD5):
        ret = True
    else:
        try:
             old_md5s = cjson.decode(open(_LXC_MD5, 'r').read())
        except cjson.DecodeError:
            ret = True
        else:
            ret = not old_md5s == md5s

    if ret:
        fh = open(_LXC_MD5, 'w')
        fh.write(cjson.encode(md5s))
        fh.close()
    return ret

