# -*- coding: utf-8 -*-
"""
fonctions communes Creole
"""
import glob
import subprocess
import os, time, sys, signal, re
from os.path import join, isfile
from pyeole.ihm import question_ouinon
from pyeole.process import system_out, system_code
from pyeole.ansiprint import print_orange
from pyeole.decorator import deprecated
from pyeole.log import init_logging
from pyeole.pkg import EolePkg
from creole.client import CreoleClient
from creole import config

from creole.i18n import _

#si creole client n'est pas démarré
global PkgManager
PkgManager = None

######################
# Gestion des noyaux #
######################

def get_wanted_kernel():
    """
    renvoie le nom du noyau sur lequel on veut tourner
    """
    ret = None
    if isfile(config.LOCALKERNEL_FILE):
        # noyau personnalisé détecté
        kernel_file = config.LOCALKERNEL_FILE
        ret = file(kernel_file).read().strip()
    else:
        kernel_images = sorted(get_installed_kernel('linux-image-'))
        current = get_current_kernel()

        # Removing kernel packges without any version
        # like linux-image-generic
        for kern in kernel_images:
            if not any(char.isdigit() for char in kern):
                kernel_images.remove(kern)
        # Get last kernel version
        last_ver = kernel_images[-1].split('-')
        if len(last_ver) >= 4:
            last_ver = "{0}-{1}-{2}".format(*kernel_images[-1].split('-')[2:5])
        elif len(last_ver) == 3 :
            last_ver = "{0}".format(last_ver[-1])

        if last_ver == current :
            ret = current
        else:
            ret = last_ver

    return ret


def get_current_kernel():
    """
    renvoie le nom du noyau sur lequel on tourne
    """
    version = system_out(['uname', '-r'])[1].strip()
    return version

def get_installed_kernel(kernel):
    """
    renvoie la liste des noyaux installés
    correspondant à celui demandé
    """
    cmd = """COLUMNS=180 dpkg -l 2>/dev/null |  awk -F " " '/^(i|h)i.*%s/ {print $2}'""" % kernel
    return os.popen(cmd).read().splitlines()

def get_eole_kernel():
    """
    renvoie le nom du noyau recommandé par EOLE
    (en dépendance de linux-image-eole)
    """
    cmd = ['dpkg-query', '-W', "-f=${Depends}", 'linux-image-eole']
    return system_out(cmd)[1].split(',')

def get_package_depends(pkg):
    """
    Renvois les dépendances d'un paquet
    """
    try:
        global PkgManager
        if PkgManager is None:
            PkgManager = EolePkg('apt')
        res = PkgManager.get_depends(pkg)
        return res
    except:
        return []

def controle_kernel():
    """
    Vérifie si on est sur le noyau désiré
    Renvoie True si un reboot est nécessaire
    """
    need_boot = False
    if isfile(config.REBOOT_FILE):
        # i.e. /var/run/reboot-required
        need_boot = True

    wanted_kernel = get_wanted_kernel()
    # on utilise le noyau spécifié
    if wanted_kernel != get_current_kernel():
        need_boot = True
        # Update grub does the job since eole-kernel-version 2.3-eole37~2
        print _(u"Updating Grub configuration")
        system_code("/usr/sbin/update-grub2")
    # reboot nécessaire ?
    return need_boot

def regen_initrd():
    """
    vérifie la présence de l'initrd
    """
    noyau = get_wanted_kernel()
    if not isfile("/boot/initrd.img-%s" % noyau):
        print _(u"Initramfs missing, generating :")
        cmd = ["/usr/sbin/update-initramfs", '-c', '-k', noyau]
        system_code(cmd)

def get_kernel_to_remove():
    """
    Obtenir la liste des noyaux a supprimer. Tous les noyaux sauf :
        - le noyau courant
        - le noyau demandé par EOLE
        - l'éventuel noyau personnalisé
    """

    to_keep_prefix = [u'linux-image-',
                      u'linux-image-extra-',
                      u'linux-signed-image-',
                      u'linux-backports-modules-',
                      u'linux-headers-',
                      u'linux-']

    # Cas particuliers :
    to_keep = [u'linux-firmware', u'linux-libc-dev']

    to_remove = []
    installed_kernels = get_installed_kernel('linux-')
    kernel_images = sorted(get_installed_kernel('linux-image-'))
    curr_kernel = get_current_kernel()
    wanted_kernel = get_wanted_kernel()
    if wanted_kernel == curr_kernel:
        wanted_kernel = None
    linux_version, pkg_version = curr_kernel.split('-')[:2]

    # Get previous kernel version
    curr_index = kernel_images.index('linux-image-{0}'.format(curr_kernel))
    prev_ver = None
    if curr_index > 0:
        prev_ver = "{0}-{1}-{2}".format(*kernel_images[curr_index-1].split('-')[2:5])
    else:
        prev_ver = "{0}-{1}-{2}".format(*kernel_images[curr_index].split('-')[2:5])

    # Get last kernel version
    last_ver = kernel_images[-1].split('-')
    if len(last_ver) >= 4:
        last_ver = "{0}-{1}-{2}".format(*kernel_images[-1].split('-')[2:5])
    elif len(last_ver) == 3 :
        last_ver = "{0}".format(last_ver[-1])

    # Generating list of kernel packages to keep
    #  - Current Running Kernel
    #  - Previous installed Kernel
    #  - Last installed Kernel
    for prefix in to_keep_prefix:
        # Keep current kernel
        cur_pkg = "{0}{1}".format(prefix, curr_kernel)
        to_keep.append(cur_pkg)
        # Keep the current kernel depends
        to_keep.extend(get_package_depends(cur_pkg))
        # Keep previous kernel
        prev_pkg = "{0}{1}".format(prefix, prev_ver)
        to_keep.append(prev_pkg)
        # Keep the previous kernel depends
        to_keep.extend(get_package_depends(prev_pkg))
        # keep Greater version kernel
        last_pkg = "{0}{1}".format(prefix, last_ver)
        to_keep.append(last_pkg)
        # Keep the last kernel depends
        to_keep.extend(get_package_depends(last_pkg))
        # Keep Wanted kernel from (defined by user)
        if wanted_kernel :
            wanted_pkg = "{0}{1}".format(prefix, wanted_kernel)
            to_keep.extend(get_package_depends(wanted_pkg))
            to_keep.append(wanted_pkg)

    # List the package version to keep
    to_keep_vers = []
    for pkg in to_keep:
        if pkg.startswith('linux-image'):
            ver = pkg.split('-')
            if len(ver) >= 4:
                try:
                    version = ver[-3:]
                    version = "{0}-{1}".format(*version[0:2])
                    if version not in to_keep_vers:
                        to_keep_vers.append(version)
                except:
                    continue

    # Looking for somme forgotten packages with a version to keep
    for pkg in installed_kernels:
        for ver in to_keep_vers:
            if ver in pkg:
                if pkg not in to_keep:
                    to_keep.append(pkg)

    to_remove = list(set(installed_kernels) - set(to_keep))
    return to_remove

def purge_rc():
    """
    Purge des paquets "rc"
    """
    cmd = """COLUMNS=180 dpkg -l|grep "^rc"|awk -F " " '{print $2}'"""
    rcs = os.popen(cmd).read().splitlines()
    for pak in rcs:
        os.system("dpkg -P %s >/dev/null" % pak)

def log(etat, msg, type_proc, console=True):
    """
    effectue un log local et éventuellement sur zephir
    """
    display = False
    log_func = 'info'
    if etat == "ERR":
        if console:
            # affichage sur la console
            display = True
        log_func = 'error'
    z_logger = init_logging(name=u'zephir', syslog=True, level=u'info', console=display)
    getattr(z_logger, log_func)("%s => %s : %s " % (type_proc, etat, msg))

def zephir(etat, msg, type_proc, console=True):
    """ gestion des messages Zephir """
    etat_zeph = None
    if etat.upper().startswith("INIT"):
        etat_zeph = -1
    elif etat.upper().startswith("FIN"):
        etat_zeph = 0
    elif etat.upper().startswith('ERR'):
        etat_zeph = 1
    elif etat.upper().startswith('MSG'):
        etat_zeph = -2
    # log local si msg ou erreur
    if (len(msg) > 0) or (etat.upper() == "ERR"):
        log(etat, msg, type_proc, console)
    # log sur zephir si disponible
    if etat_zeph is not None:
        try:
            # si serveur enregistré, on envoie un log à Zéphir
            from zephir.zephir_conf.zephir_conf import id_serveur
        except:
            pass
        else:
            from zephir.lib_zephir import log as zlog
            zlog(type_proc, etat_zeph, msg, str(time.ctime()))

def init_proc(type_proc):
    """
    initialisation d'une procédure (log démarrage + bcage éventuel)
    """
    if verify_lock(type_proc):
        return True
    else:
        return False

#def end_proc(etat,msg,type_proc):
#    """
#    loggue la fin d'une procédure
#    """
#     log(etat,msg,type_proc )

def verify_lock(name):
    """
    vérifie le bloquage ou non d'une  procédure
    """
    LOCK_FILE = "/usr/share/zephir/zephir_locks"
    if name == "":
        return True
    from zephir.lib_zephir import zephir_path
    try:
        from zephir.lib_zephir import config, zephir, convert
        locks = convert(zephir.serveurs.get_locks(config.id_serveur))
        if locks[0] == 0:
            # erreur sur zephir, on ignore cette phase
            raise Exception
        locks = [lock[0] for lock in locks[1]]
    except Exception, mess:
        # pas de message d'erreur si le serveur n'est pas enregistré
        zephir_error = False
        if isfile(join(zephir_path, "zephir_conf", "zephir_conf.py")):
            # on ne bloque pas si l'appel à zephir échoue
            print ""
            print_orange(_(u"Checking permissions on Zéphir for {0} impossible.").format(name))
            print_orange(_(u"Error message: {0}").format(mess))
            zephir_error = True
        # on regarde le denier état
        if os.path.exists(LOCK_FILE):
            if zephir_error:
                print_orange(_(u"Using stored parameters"))
            file_lock = file(LOCK_FILE)
            locks = file_lock.read().split('\n')
            file_lock.close()
            # on bloque si interdit
            if name in locks:
                return False
    else:
        # mise en place du fichier de droits
        content = "\n".join(locks)
        try:
            file_lock = file(LOCK_FILE, "w")
            file_lock.write(content)
            file_lock.close()
        except:
            print _(u"Updating {0} impossible (insufficient rights).").format(LOCK_FILE)
        # retour du code
        if name in locks:
            return False
    return True

def register_module(ask=False):
    """
    Enregistrement Eole
    """
    system_out("/usr/share/creole/diag.py")
    tool_dir = '/usr/share/register'
    if not os.path.isdir(tool_dir):
        # eole-register pas installé ?
        return
    if not isfile(join(tool_dir,'.registerok')):
        # pas d'enregistrement préalable
        if not ask:
            return
        print
        print _(u"http://eole.ac-dijon.fr/materiel/ allows you to consult compatible hardware database.")
        if question_ouinon(_(u"Do you want to send this server hardware description to improve compatible hardware database."), default=_(u'oui')) == _(u'non'):
            return
        file(join(tool_dir, '.registerok'), 'w').close()

    sys.path.append('/usr/share/')
    from register import register
    os.chdir(tool_dir)
    register.initialize()

def proper_exit(*args):
    """callback à appeler lors d'un ctrl-c"""
    print(_(u"Cancelled by user."))
    os._exit(1)

def catch_signal(exit_func=None):
    """gestion du ctrl-c"""
    if not exit_func:
        signal.signal(signal.SIGINT, proper_exit)
    else:
        signal.signal(signal.SIGINT, exit_func)

